@@ -5,19 +5,18 @@ loonPlotFactory <- function(factory_tclcmd, factory_path, factory_window_title="
         new.toplevel <- TRUE
         parent <- l_toplevel()
     }
-    
+
     child <- l_subwin(parent, factory_path)
-    
     plot <- try(tcl(factory_tclcmd, child, ...))
-    
+
     if(is(plot, 'try-error')) {
         if(new.toplevel) tkdestroy(parent)
         stop(paste0(factory_window_title, " could not be created."))
     }
-    
+
     if(new.toplevel) {
         tkpack(plot,  fill="both", expand=TRUE)
-        tktitle(parent) <- paste(factory_window_title, plot)
+        tktitle(parent) <- paste(factory_window_title, "--path:", plot)
         ## Bind the toplevel <FocusIn> event to update the loon inspector
         tcl("bind", parent, "<FocusIn>",
             paste0("+::loon::setLoonInspectorActiveWidget ", plot))
@@ -28,12 +27,12 @@ loonPlotFactory <- function(factory_tclcmd, factory_path, factory_window_title="
 
         tcl("bind", parent, "<Control-KeyPress-P>",
             function()exportImageDialog(plot))
-        
+
     }
-    
+
     plot <- as.character(plot)
-    
+
     class(plot) <- "loon"
-    
+
     return(plot)
 }
